// <Title>Butch Cassidy and the Sundance Kid</Title>
// <Description>Two outlaws (Paul Newman, Robert Redford) romp through Wyoming, then sail to Bolivia and take on an army.</Description>
// <Duration>112 min</Duration>
// <Category>Movie</Category>
// <SubCategory>Western</SubCategory>
// <Starring>Paul Newman, Robert Redford, Katharine Ross, Strother Martin, Henry Jones, Jeff Corey, George Furth, Cloris Leachman, Ted Cassidy, Kenneth Mars, Donnelly Rhodes, Jody Gilbert, Timothy Scott, Don Keefer, Charles Dierkop, Francisco Cordova, Nelson Olmstead, Paul Bryar, Sam Elliott, Claude Akins</Starring>
// <Director>George Roy Hill</Director>
// <Producer>Paul Monash, John Foreman</Producer>
// <Rated>PG</Rated>
// <ExpandedRatingsList>Adult Situations, Language and Mild Violence</ExpandedRatingsList>
// <Year>1969</Year>
// <MiscList>****</MiscList>
// <ExternalID>MV0006560000</ExternalID>
// <Language>English</Language>
// <StationID>MPLEX</StationID>
// <StartTime>2004-03-13T09:45:00</StartTime>
// <StopTime>2004-03-13T11:40:00</StopTime>

import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.xml.sax.*;
import java.util.StringTokenizer;
import java.util.regex.*;

public class xml2my
{
    public static String version="1.1.0";

    // History:
    // 1.0.0 -- 11 Jul 2004 
    //
    // v 1.1.0 14-July 2004
    // -- create multiple my files using episode name
    // -- create output file in same location as source
    // -- add batch file to allow drag-and-drop conversion

    public static void print_usage() {
	System.out.println("xml2hava -- convert malore XML to mlbdude my -- version "+version);
	System.out.println("Usage:");
	System.out.println("   xml2java xml_file [outputfile]");
	System.out.println("    outputfile defaults to episodename.my");
    }
    
    public static void main(String[] args) {
	
	System.out.println(args.length);
	if (args.length <1 || args.length >2) 
	{
	    print_usage();
	    return;
	}

	String input_file=args[0];
	String output_location=null;
	String output_file=null;

	if (args.length==2) {
	    output_file=args[1];
	} else {
	    if ( input_file.indexOf("\\") != -1 )
	    {
		output_location=input_file.substring(0,input_file.lastIndexOf("\\")+1);
	    }
	    else
	    {
		output_location="";
	    }
	}

	Document doc = parseXmlFile(input_file,false);
	try
	{
	    // attempt to catch major errors

	    if (  doc==null ) {
		System.out.println(
		    "Failed to read XML from "+input_file);
		return;
	    }
	    
	    // got to here, try to parse
	    Element topElement = doc.getDocumentElement();

	    // Shows.xml files are <show> elements in a <tv> tag
	    // whereas file.xml files are a single <show> element
	    
	    NodeList TV=doc.getElementsByTagName("TV");
	    NodeList Shows;
	    if ( TV.getLength() > 0) 
		Shows=((Element)TV.item(0)).getElementsByTagName("Show");
	    else {
		Shows=doc.getElementsByTagName("Show");
	    }
	    
	    if ( Shows == null || Shows.getLength() == 0) {
		System.out.println("Failed to find <Shows> in XML");
	    }
	    else {
		FileWriter fw = null;
		BufferedWriter bw =null;
		PrintWriter out = null;

		if ( output_file!=null) {
		    System.out.println("Creating "+output_file);
		    fw = new FileWriter(output_file);
		    bw = new BufferedWriter(fw);
		    out = new PrintWriter(bw);	    
		}

		for(int shownum=0; shownum<Shows.getLength(); shownum++) {
		 
		    Element show=(Element)Shows.item(shownum);

		    String title=getXML(show,"Title");
		    String subtitle=getXML(show,"Episode");

		    if ( output_file==null)
		    {
			// multiple output files
			if ( out != null )
			{
			    // close and re-open output file
			    out.close();
			    try
			    {
				bw.close();
				fw.close();
			    }
			    catch (Exception writeClose)
			    {
			    }
			}
			String filename=null;
			if ( title.length()==0)
			    filename="info"+Integer.toString(shownum+1);
			else
			{
			    filename=title;
			    if ( subtitle.length()==0 ) {
				if ( Shows.getLength() >1 )
				    filename+=" - " + Integer.toString(shownum+1);
			    } else {
				filename+=" - " + subtitle;
			    }
			}
			// clean filename string
			filename=filename.replaceAll("[^a-zA-Z0-9 _\\-]","_");
			// add path
			filename=output_location+filename+".my";
			
			//open it
			System.out.println("Creating "+filename);
			fw = new FileWriter(filename);
			bw = new BufferedWriter(fw);
			out = new PrintWriter(bw);	    
			out.println("[1]");
  		    }
		    else
		    {
			// single file, possibly multiple entries
			out.println("["+Integer.toString(shownum+1)+"]");
		    }
		    
		    out.println("title="+title);
		    if ( subtitle.length()>0 )
			out.println("subtitle="+subtitle);
		    out.println("overview="+getXML(show,"Description"));
		    out.println("produced="+getXML(show,"Year"));
		    out.println("duration="+getXML(show,"Duration"));
		    out.println("rating="+getXML(show,"Rated"));
		    out.print("category="+getXML(show,"Category"));
		    String SubCategory=getXML(show,"SubCategory");
		    if ( SubCategory.length()>0)
			out.print("/"+SubCategory);
		    out.println();
		    String Director=getXML(show,"Director");	    
		    if ( Director.length()>0)
			out.println("people=Director,\""+Director+"\"");
		    String actors[]=getXML(show,"Starring").split(", *");
		    if ( actors.length>0 && actors[0].length()>0) {
			out.print("people=Principals");
			for (int actor=0; actor<actors.length; actor++)
			    out.print(",\""+actors[actor]+"\"");
			out.println();
		    }
		    out.println();
		    out.println();
		}
		out.close();
		
		try
		{
		    bw.close();
		    fw.close();
		}
		catch (Exception writeClose)
		{
		}
	    }
	}
	catch (Exception e)
	{
	    System.out.println("Problem parsing XML:");
	    e.printStackTrace(System.out);
	    return ;		
	}
	
    }
    //This method gets the xml out of the document based on the tag and the parent
    //Caveat, it goes off the first match -- limit the matches using element selection
    public static String getXML(Element doc, String tag) {
	String tmpstr = null;
	try {
	    NodeList tmpNodes = doc.getElementsByTagName(tag);
	    
	    //get the first match
	    for (int i = 0; i < tmpNodes.getLength() && tmpstr==null; i++)
	    {
//		System.out.println(i);
//		if ((((Element)(tmpNodes.item(i).getParentNode())).getTagName()).equals(parent))
		{
		    // check children for value
//		    System.out.println("got an  xml node  for "+parent+"/"+tag);
//		    System.out.println("Node: "+((Element)tmpNodes.item(i)).getTagName());
//		    System.out.println("Node text: "+tmpNodes.item(i).toString());

		    NodeList children=tmpNodes.item(i).getChildNodes();
		    for ( int child=0; child < children.getLength(); child++) {
			if ( children.item(child).getNodeType()==Node.TEXT_NODE ) {
//			    System.out.println("value" +children.item(child).getNodeValue());
			    tmpstr=children.item(child).getNodeValue();
			    break;
			}
		    }
		}
	    }
	    
	}
	catch (Exception e) {
	    System.out.println("Failed to get xml value for "+tag);
	    e.printStackTrace();
	}

	if (tmpstr ==null)
	    // found nothing, return blank
	    return "";

	// clean up string
	
	tmpstr.trim();
	
	
	//trim didn't do its job
	if ((int)tmpstr.charAt(0) == 32)
	    tmpstr = tmpstr.substring(1);
	
	if ((int)tmpstr.charAt(tmpstr.length() -1) == 32)
	    tmpstr = tmpstr.substring(0, tmpstr.length() - 1);
	
	return tmpstr;
    }


    // Source for sample: http://javaalmanac.com/egs/javax.xml.transform/WriteDom.html
    // Parses an XML file and returns a DOM document.
    // If validating is true, the contents is validated against the DTD
    // specified in the file.
    public static Document parseXmlFile(String xml_url, boolean validating) {
	try {
	    // Create a builder factory
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    factory.setValidating(validating);
	    
	    // Create the builder and parse the file
	    Document doc = factory.newDocumentBuilder().parse(xml_url);
	    return doc;
	} 
	catch (SAXException e) 
	{
	    // A parsing error occurred; the xml input is not valid
	    System.out.println("Error parsing the xml, the input is not valid: " + e);
	    e.printStackTrace();
	} 
	catch (ParserConfigurationException e) 
	{
	    System.out.println("An unexpected ParserConfigurationException occured: " + e);
	    e.printStackTrace();
	} 
	catch (IOException e) 
	{
	    System.out.println("An unexpected IO error occured: " + e);
	    e.printStackTrace();
	}
	return null;
    }
}
    
    
	
	
